#include <iostream>
#include "exception.h"
#include "Vector.h"
#include <iomanip>
using namespace std;


CSLRMatrix::CSLRMatrix(double* p_adiag, double* p_altr, size_t* p_jptr, size_t* p_iptr, size_t p_diag, size_t p_elem) :
    m_adiag(new double[p_diag]),
    m_diag(p_diag),
    m_altr(new double[p_elem]),
    m_elem(p_elem),
    m_jptr(new size_t[p_elem]),
    m_iptr(new size_t[p_diag + 1])
{
    for (int i = 0; i < p_diag; ++i)
    {
        m_adiag[i] = p_adiag[i];
    }
    for (int i = 0; i < p_elem; ++i)
    {
        m_altr[i] = p_altr[i];
        m_jptr[i] = p_jptr[i];
    }
    for (int i = 0; i < p_diag + 1; ++i)
    {
        m_iptr[i] = p_iptr[i];
    }
}


CSLRMatrix::CSLRMatrix(const CSLRMatrix& p_mat) :
    m_adiag(new double[p_mat.m_diag]),
    m_diag(p_mat.m_diag),
    m_altr(new double[p_mat.m_elem]),
    m_elem(p_mat.m_elem),
    m_jptr(new size_t[p_mat.m_elem]),
    m_iptr(new size_t[p_mat.m_diag + 1])
{
    for (int i = 0; i < p_mat.m_diag; ++i)
    {
        m_adiag[i] = p_mat.m_adiag[i];
    }
    for (int i = 0; i < p_mat.m_elem; ++i)
    {
        m_altr[i] = p_mat.m_altr[i];
        m_jptr[i] = p_mat.m_jptr[i];
    }
    for (int i = 0; i < p_mat.m_diag + 1; ++i)
    {
        m_iptr[i] = p_mat.m_iptr[i];
    }
}


CSLRMatrix::CSLRMatrix(CSLRMatrix&& p_mat) :
    m_adiag(p_mat.m_adiag),
    m_diag(p_mat.m_diag),
    m_altr(p_mat.m_altr),
    m_elem(p_mat.m_elem),
    m_jptr(p_mat.m_jptr),
    m_iptr(p_mat.m_iptr)
{
    p_mat.m_adiag = nullptr;
    p_mat.m_altr = nullptr;
    p_mat.m_jptr = nullptr;
    p_mat.m_iptr = nullptr;
    p_mat.m_diag = 0;
    p_mat.m_elem = 0;
}


CSLRMatrix::~CSLRMatrix() {
    delete[] m_adiag;
    delete[] m_altr;
    delete[] m_iptr;
    delete[] m_jptr;
}


std::istream& operator>>(std::istream& p_is, CSLRMatrix& p_mat) {

    printf("\nenter the elements that are on the main diagonal\n");
    for (int i = 0; i < p_mat.m_diag; i++)
        p_is >> p_mat.m_adiag[i];
    printf("\nenter nonzero matrix elements\n");
    for (int i = 0; i < p_mat.m_elem; i++)
        p_is >> p_mat.m_altr[i];
    printf("\nin which columns are these elements\n");
    for (int i = 0; i < p_mat.m_elem; i++)
        p_is >> p_mat.m_jptr[i];
    printf("\nWith what position in the array of all non - zero elements does the i-th line begin matrices\n");
    for (int i = 0; i < p_mat.m_diag+1; i++)
        p_is >> p_mat.m_iptr[i];
    return p_is;
}


std::ostream& operator<<(std::ostream& p_os, const CSLRMatrix& p_mat) {
    for (int i = 0; i < p_mat.m_diag; i++) {
        for (int j = 0; j < p_mat.m_diag; j++) {
            double val = 0;
            if (i == j)
            {
                val = p_mat.m_adiag[i];
            }
            else
            {
                for (int k = p_mat.m_iptr[i] - 1; k < p_mat.m_iptr[i + 1] - 1; k++) {
                    if (p_mat.m_jptr[k] - 1 == j) {
                        val = p_mat.m_altr[k];
                        break;
                    }
                }
                for (int k = p_mat.m_iptr[j] - 1; k < p_mat.m_iptr[j + 1] - 1; k++) {
                    if (p_mat.m_jptr[k] - 1 == i) {
                        val = p_mat.m_altr[k];
                        break;
                    }
                }
            }
            p_os << setw(4) << val;
        }
        p_os << std::endl << std::endl;
    }
    return p_os;
}


size_t CSLRMatrix::matsize() const
{
    return m_diag;
}


size_t CSLRMatrix::nzn() const
{
    int ans = m_elem * 2;
    for (int i = 0; i < m_diag; i++)
    {
        if (m_adiag[i] != 0) ans++;
    }
    return ans;
}


const CSLRMatrix& CSLRMatrix::operator=(const CSLRMatrix& p_mat)
{
    if (this == &p_mat)
    {
        return *this;
    }
    if (m_diag != p_mat.m_diag)
    {
        delete[] m_adiag;
        delete[] m_altr;
        delete[] m_jptr;
        delete[] m_iptr;
        m_adiag = new double[p_mat.m_diag];
        m_altr = new double[p_mat.m_elem];
        m_jptr = new size_t[p_mat.m_elem];
        m_iptr = new size_t[p_mat.m_diag+ 1];
        m_diag = p_mat.m_diag;
        m_elem = p_mat.m_elem;
    }
    for (int i = 0; i < p_mat.m_diag; ++i)
    {
        m_adiag[i] = p_mat.m_adiag[i];
    }
    for (int i = 0; i < p_mat.m_elem; ++i)
    {
        m_altr[i] = p_mat.m_altr[i];
        m_jptr[i] = p_mat.m_jptr[i];
    }
    for (int i = 0; i < p_mat.m_diag + 1; ++i)
    {
        m_iptr[i] = p_mat.m_iptr[i];
    }
    return *this;
}


CSLRMatrix& CSLRMatrix::operator=(CSLRMatrix&& p_mat)
{
    delete[] m_adiag;
    delete[] m_altr;
    delete[] m_jptr;
    delete[] m_iptr;
    m_adiag = p_mat.m_adiag;
    m_altr = p_mat.m_altr;
    m_jptr = p_mat.m_jptr;
    m_iptr = p_mat.m_iptr;
    m_diag = p_mat.m_diag;
    m_elem = p_mat.m_elem;
    p_mat.m_adiag = nullptr;
    p_mat.m_altr = nullptr;
    p_mat.m_jptr = nullptr;
    p_mat.m_iptr = nullptr;
    p_mat.m_diag = 0;
    p_mat.m_elem = 0;
    return *this;
}

Vector operator*(const CSLRMatrix& p_mat, const Vector& p_vec)
{
    if (p_mat.matsize() != p_vec.dimension())
    {
        throw IncompatibleDimException{};
    }
    Vector res(p_vec);
    for (int i = 0; i < p_mat.matsize(); i++)
    {
        res[i] = p_vec[i] * p_mat.m_adiag[i];
    }
    for (int i = 0; i < p_mat.matsize(); i++)
    {
        for (int j = p_mat.m_iptr[i] - 1; j < p_mat.m_iptr[i + 1] - 1; j++)
        {
            res[i] += p_vec[p_mat.m_jptr[j] - 1] * p_mat.m_altr[j];
            res[p_mat.m_jptr[j] - 1] += p_vec[i] * p_mat.m_altr[j];
        }
    }
    return res;
}


CSLRMatrix operator*(const CSLRMatrix& p_mat, double p_scalar)
{
    CSLRMatrix result(p_mat);
    for (int i = 0; i <p_mat.m_diag; i++)
    {
        result.m_adiag[i] *= p_scalar;
    }
    for (int i = 0; i < p_mat.m_elem; i++)
    {
        result.m_altr[i] *= p_scalar;
    }
    return result;
}


CSLRMatrix operator*(double p_scalar, const CSLRMatrix& p_mat)
{
    return p_mat * p_scalar;
}