#include <iostream>
#include "exception.h"
#include <iomanip>
#include "Vector.h"
#include "Matrix.h"


Vector::Vector(int p_n, double p_value) {
    m_size = p_n;
    m_data = new double[m_size];
    for (int i = 0; i < m_size; i++) {
        m_data[i] = p_value;
    }
}


Vector::Vector(const Vector& p_vec):
    m_size ( p_vec.m_size),
    m_data ( new double[m_size])
{
    
    for (int i = 0; i < m_size; i++) {
        m_data[i] = p_vec.m_data[i];
    }
}


Vector::Vector(Vector&& p_vec):
    m_size(p_vec.m_size),
    m_data(p_vec.m_data)
{
    p_vec.m_data = nullptr;
    p_vec.m_size = 0;
}


Vector::~Vector() {
    delete[] m_data;
}


Vector& Vector:: operator=(const Vector& p_vec) {
    if (this != &p_vec) {
        delete[] m_data;
        m_size = p_vec.m_size;
        m_data = new double[m_size];
        for (int i = 0; i < m_size; i++) {
            m_data[i] = p_vec.m_data[i];
        }
    }
    return *this;
}


Vector& Vector:: operator=(Vector&& p_vec) {
    delete[] m_data;
    m_size = p_vec.m_size;
    m_data = p_vec.m_data;
    p_vec.m_data = nullptr;
    p_vec.m_size = 0;
    return *this;
}


Vector& Vector::operator+=(const Vector& p_vec) {
    if (m_size != p_vec.m_size) {
        throw IncompatibleDimException();
    }
    for (int i = 0; i < m_size; i++) {
        m_data[i] += p_vec.m_data[i];
    }
    return *this;
}


Vector& Vector::operator-=(const Vector& p_vec) {
    if (m_size != p_vec.m_size) {
        throw IncompatibleDimException();
    }
    for (int i = 0; i < m_size; i++) {
        m_data[i] -= p_vec.m_data[i];
    }
    return *this;
}


Vector Vector::operator+() const {
    return *this;
}


Vector Vector::operator-() const {
    Vector result(m_size);
    for (int i = 0; i < m_size; i++) {
        result.m_data[i] = -m_data[i];
    }
    return result;
}


Vector operator+(const Vector& p_vec_1, const Vector& p_vec_2) {
    if (p_vec_1.m_size != p_vec_2.m_size) {
        throw IncompatibleDimException();
    }
    Vector result(p_vec_1);
    result += p_vec_2;
    return result;
}


Vector operator-(const Vector& p_vec_1, const Vector& p_vec_2) {
    if (p_vec_1.m_size != p_vec_2.m_size) {
        throw IncompatibleDimException();
    }
    Vector result(p_vec_1);
    result -= p_vec_2;
    return result;
}


double operator*(const Vector& p_vec_1, const Vector& p_vec_2) {
    double result = 0.0;
    for (int i = 0; i < p_vec_1.m_size; i++) {
        result += p_vec_1.m_data[i] * p_vec_2.m_data[i];
    }
    return result;
}

Vector operator*(const Vector& p_vec, double p_scalar) {
    Vector result(p_vec);
    for (int i = 0; i < p_vec.m_size; i++) {
        result.m_data[i] *= p_scalar;
    }
    return result;
}


Vector operator*(double p_scalar, const Vector& p_vec) {
    return p_vec * p_scalar;
}


double& Vector::operator[](int p_index) {
    if (p_index < 0 || p_index >= m_size) {
        throw OutOfRangeException();
    }
    return m_data[p_index];
}


const double& Vector::operator[](int p_index) const {
    if (p_index < 0 || p_index >= m_size) {
        throw OutOfRangeException();
    }
    return m_data[p_index];
}


std::ostream& operator<<(std::ostream& p_os, const Vector& p_vec) {
    p_os << "[";
    for (int i = 0; i < p_vec.m_size; i++) {
        p_os << p_vec.m_data[i];
        if (i < p_vec.m_size - 1) {
            p_os << ", ";
        }
    }
    p_os << "]";
    return p_os;
}


std::istream& operator>>(std::istream& p_is, Vector& p_vec) {
    for (int i = 0; i < p_vec.m_size; i++) {
        p_is >> p_vec.m_data[i];
    }
    return p_is;
}


Vector:: operator double* () {
    return m_data;
}


double Vector::length() const {
    double result = 0.0;
    for (int i = 0; i < m_size; i++) {
        result += m_data[i] * m_data[i];
    }
    return sqrt(result);
}


int Vector::dimension() const {
    return m_size;
}