#pragma once
#include <cstdint>
#include <iostream>
#include <exception>
#include "Matrix.h"
class CSLRMatrix;
class Vector
{

public:

	explicit Vector(int p_n = 0, double p_value = 0.0);

	Vector(const Vector& p_vec);

	~Vector();

	Vector(Vector&& p_vec);

	double& operator[](int p_index);

	const double& operator[](int p_index) const;

	Vector& operator=(const Vector& p_vec);
	Vector& operator=(Vector&& p_vec);

	Vector& operator+=(const Vector& p_vec);
	Vector& operator-=(const Vector& p_vec);

	Vector operator+() const;
	Vector operator-() const;

	friend Vector operator+(const Vector& p_vec_1, const Vector& p_vec_2);
	friend Vector operator-(const Vector& p_vec_1, const Vector& p_vec_2);

	friend double operator*(const Vector& p_vec_1, const Vector& p_vec_2);
	friend Vector operator*(const Vector& p_vec, double p_scalar);
	friend Vector operator*(double p_scalar, const Vector& p_vec);

	friend std::ostream& operator<<(std::ostream& p_os, const Vector& p_vec);
	friend std::istream& operator>>(std::istream& p_is, Vector& p_vec);

	explicit operator double* ();

	double length() const;
	int dimension() const;

private:
	int m_size;
	double* m_data;
	friend class CSLRMatrix;
};






